@@ -1,7 +1,7 @@
 # Provides an example of a self-navigating algorithm which you are under no obligation to use
 from microprediction.writer import MicroWriter
-from microprediction.reader import default_url
-import muid, random, time
+from microconventions import api_url
+import random, time
 from collections import OrderedDict
 from microprediction.samplers import exponential_bootstrap, approx_mode
 import pprint
@@ -10,7 +10,7 @@ from statistics import median
 
 
 class MicroCrawler(MicroWriter):
-    """To create a crawler you can derive from this class (if you want) and it is suggested you overwrite one of more of the following methods:
+    """To create a crawler you can derive from this class and it is suggested you overwrite one of more of the following methods:
 
            sample            - Method that creates predictions
            downtown          - Stuff to do when there is nothing terribly urgent
@@ -18,7 +18,7 @@ class MicroCrawler(MicroWriter):
         You might also want to change
 
            candidate_streams  - A list of streams you want your crawler to look at
-           candidate_delays   - A list of horizons to predict (from the set self.delays)
+           candidate_delays   - A list of horizons to predict (from the set self.DELAYS)
            update_frequency   - How many times to predict per arriving data point
 
        Good luck!
@@ -54,7 +54,7 @@ class MicroCrawler(MicroWriter):
 
             :returns [ int ]    List of horizons your bot is willing to participate in
         """
-        return self.delays
+        return self.DELAYS
 
     def update_frequency(self, name=None, delay=None):
         """ Determines how often to update samples """
@@ -77,12 +77,15 @@ class MicroCrawler(MicroWriter):
         assert len(scenarios)==self.num_predictions, "Your sammpler should product a vector of length "+str(self.num_predictions)
         return sorted(scenarios)
 
-
     def downtime(self,seconds,**ignored):
         """ This will be called when a small gap opens up between times when you need to predict """
         # Be careful if you override it or you might be late for your next prediction
         time.sleep(seconds)
 
+    def withdrawing(self,horizon):
+        """ Teardown when you withdraw from a horizon """
+        pass
+
     #############################################################
     #   Typically don't want to override the rest of these      #
     #############################################################
@@ -90,8 +93,8 @@ class MicroCrawler(MicroWriter):
 
     def __init__(self, write_key=None, base_url=None, verbose=False, quietude=50, stop_loss=10, min_budget=0., max_budget=10, min_lags = 25, max_lags=1000000, sponsor_min=12, sleep_time=300):
         """  """
-        super().__init__(base_url=base_url or default_url(), write_key=write_key, verbose=verbose )
-        assert muid.difficulty(write_key) >= 8, "Invalid write_key for crawler. See www.muid.org to mine one. "
+        super().__init__(base_url=base_url or api_url(), write_key=write_key, verbose=verbose )
+        assert self.key_difficulty(write_key) >= 7, "Invalid write_key for crawler. See www.muid.org to mine one. "
         assert not self.base_url[-1]=='/','Base url should not have trailing /'
         assert stop_loss>0,"Stop loss must be positive "
         self.quietude    = int(quietude)       # e.g. if set to 10, will only print 1/10th of the time
@@ -103,7 +106,7 @@ class MicroCrawler(MicroWriter):
         self.min_lags    = min_lags            # Insist on historical data
         self.max_lags    = max_lags
         self.performance = self.get_performance()
-        self.active      = self.get_active()
+        self.active      = self.get_active()   # List of active horizons
         self.horizon_blacklist = list()
         self.cancelled   = list()
         self.next_prediction_time = dict()
@@ -134,6 +137,10 @@ class MicroCrawler(MicroWriter):
             self.seconds_until_next = min(upcoming)
         except:
             self.seconds_until_next = 10*60
+        seconds_until_quit = self.end_time-time.time()
+        if int(seconds_until_quit) in [1000,100,10,5]:
+            print('Seconds until quit: '+str(seconds_until_quit),flush=True)
+        self.seconds_until_next = min(self.seconds_until_next,seconds_until_quit)
         return self.seconds_until_next
 
     def __repr__(self):
@@ -175,6 +182,7 @@ class MicroCrawler(MicroWriter):
         self.horizon_blacklist.append(horizon)
         self.active      = self.get_active()
         self.performance = self.get_performance()
+        self.withdrawing(horizon)
 
     def withdraw_from_worst(self, num=1):
         horizons = self.worst_active_horizons()[:num]
@@ -192,16 +200,18 @@ class MicroCrawler(MicroWriter):
                 delays = self.candidate_delays(name=name)
                 for delay in delays:
                     horizon = self.horizon_name(name=name,delay=delay)
-                    if not horizon in self.horizon_blacklist:
-                        if not (horizon in self.next_prediction_time) or time.time()>self.next_prediction_time[horizon]:
-                            horizons.append(horizon)
+                    if not horizon in self.active:
+                        if not horizon in self.horizon_blacklist:
+                            if not (horizon in self.next_prediction_time) or time.time()>self.next_prediction_time[horizon]:
+                                horizons.append(horizon)
             if horizons:
                 return random.choice(horizons)
 
     def estimate_prediction_time(self, lagged_times, delay, num_intervals):
         # Base on lagged times of observations and the horizon, set a time at which to next create a prediction
         if len(lagged_times) > 5:
-            dt = -approx_mode(np.diff(lagged_times))
+            neg_dt = approx_mode(np.diff(lagged_times))
+            dt = -neg_dt if neg_dt is not None else None
             if dt is None or self.sleep_time > dt:
                 dt = self.sleep_time
             # e.g. dt=90 and delay=310  we have 80 seconds grace
@@ -225,7 +235,7 @@ class MicroCrawler(MicroWriter):
         """ Given a stream and horizon, try to submit predictions """
         horizon = self.horizon_name(name=name, delay=delay)
         lagged_values = self.get_lagged_values(name)
-        exec = 0
+        execut = 0
         message = {'name':name,'delay':delay,'submitted':False}
         if len(lagged_values) > self.max_lags:
             message.update({"reason": "Too many lags - blacklisting", "lagged_len": len(lagged_values)})
@@ -247,8 +257,8 @@ class MicroCrawler(MicroWriter):
             # Call self.sample() but first let it know how much time it has
             self.update_seconds_until_next(exclude=[horizon])
             scenario_values = self.sample(lagged_values=lagged_values,lagged_times=lagged_times, name=name, delay=delay )
-            exec = self.submit(name=name, values=scenario_values, delay=delay)
-            message.update({'submitted':True,'exec':exec})
+            execut = self.submit(name=name, values=scenario_values, delay=delay)
+            message.update({'submitted':True,'exec':execut})
             message.update({"median": median(scenario_values),
                             "mean": np.mean(scenario_values),
                             "std": np.std(scenario_values),
@@ -272,7 +282,7 @@ class MicroCrawler(MicroWriter):
                     self.backoff[horizon] = delay
                 message.update({'backoff':self.backoff[horizon]})
 
-            if not exec:
+            if not execut:
                 message.update({"submitted":False,"reason":"execution failure","confirms":self.get_confirms()[-1:], "errors": self.get_errors()[-1:]})
                 print("---------- ERROR ------------")
                 pprint.pprint(message)
@@ -284,27 +294,32 @@ class MicroCrawler(MicroWriter):
             print(" ")
             pprint.pprint(message)
             print(" ", flush=True)
-        return exec
+        return execut
 
     def initial_urgency_multiplier(self,horizon):
         """ Seconds """
         return 20 if '~' in horizon else 1
 
-    def run(self):
+    def run(self,timeout=None):
         # Pick up where we left off, but stagger
         self.performance = self.get_performance()
         self.active = self.get_active()
+        self.start_time = time.time()
+        self.end_time = time.time()+timeout if timeout is not None else time.time()+10000000
         print('Currently predicting for ' + str(len(self.active)) + ' horizons')
         self.stream_candidates = self.candidate_streams()
         print('Found ' + str(len(self.stream_candidates)) + ' candidate streams.', flush=True)
-        self.next_prediction_time = dict( [ (horizon, time.time() + k*self.initial_urgency_multiplier(horizon)) for k,horizon in enumerate(self.active)])
+        desired_streams = [self.horizon_name(stream_name, horizon) for stream_name in self.stream_candidates for horizon in [70, 310, 910]]
+        self.next_prediction_time = dict( [ (stream, time.time() + k*self.initial_urgency_multiplier(stream)) \
+            for k, stream in enumerate(self.active) if stream in desired_streams])
+
 
         print("---------- Restarting -------------")
         pprint.pprint(self.__repr__())
         print("-----------------------------------",flush=True)
 
 
-        while True:
+        while time.time()<self.end_time:
 
             # Periodically consider entering new horizons and withdrawing from others
             self.update_seconds_until_next()
@@ -320,7 +335,7 @@ class MicroCrawler(MicroWriter):
 
             # If there is time consider entering a new stream, but not too often
             self.update_seconds_until_next()
-            if (self.seconds_until_next>np.random.rand()*500) or (len(self.active)==0):
+            if (self.seconds_until_next>np.random.rand()*500) or (len(self.active)<10):
                 if (time.time()-self.last_new_horizon>10*60) or (len(self.active)==0):
                     horizon = self.next_horizon(exclude=self.horizon_blacklist)
                     if horizon:
