@@ -1,18 +1,19 @@
-from microprediction.reader import MicroReader, default_url
-import muid, requests, pprint
+from microprediction.reader import MicroReader
+from microconventions import api_url
+import requests, pprint
 
 class MicroWriter(MicroReader):
 
     def __init__(self, write_key="invalid_key", base_url=None, verbose=True, **kwargs ):
         """ Create the ability to write """
-        super().__init__(base_url=base_url or default_url(),**kwargs)
+        super().__init__(base_url=base_url or api_url(),**kwargs)
         string_write_key = write_key if isinstance(write_key,str) else write_key.decode()
-        assert muid.validate(string_write_key), "Invalid write_key. Mine one at muid.org. "
+        assert self.key_difficulty(string_write_key), "Invalid write_key. Mine one at muid.org. "
         self.write_key = string_write_key
         self.verbose   = verbose
 
     def __repr__(self):
-        return {'write_key':self.write_key,"animal":muid.animal(self.write_key)}
+        return {'write_key':self.write_key,"animal":self.animal_from_key(self.write_key)}
 
     def get_home(self):
         res = requests.put(self.base_url + '/live/' + self.write_key )
@@ -75,6 +76,35 @@ class MicroWriter(MicroReader):
         else:
             raise Exception('Failed for ' + self.write_key)
 
+    def put_balance(self, source_write_key, amount=100. ):
+        """ Transfer some balance into self.write_key """
+        res = requests.put(self.base_url + '/balance/' + self.write_key, data={"source_write_key": source_write_key,"amount":amount})
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
+    def donate_balance(self, recipient_write_key, amount=100.):
+        """ Give some of your balance to a key that has a negative balance """
+        res = requests.put(self.base_url + '/balance/' + recipient_write_key,
+                           data={"source_write_key": self.write_key, "amount": amount})
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
+    def bolster_balance_by_mining(self,seconds=1):
+        """ For a short time, try to burn a MUID to bolster flagging balance """
+        key = self.maybe_create_key(difficulty=12,seconds=seconds)
+        if key is not None:
+            self.put_balance(source_write_key=key)
+            return key
+
+    def restore_balance_by_mining(self, difficulty=12):
+        """ Mine a MUID and deposit it to bring the balance up from a negative number """
+        source_write_key = self.create_key(difficulty=difficulty)
+        return self.put_balance(source_write_key=source_write_key)
+
     def get_confirms(self):
         res = requests.get(self.base_url + '/confirms/' + self.write_key)
         if res.status_code == 200:
@@ -110,6 +140,13 @@ class MicroWriter(MicroReader):
         else:
             raise Exception('Failed for ' + self.write_key)
 
+    def delete_performance(self):
+        res = requests.delete(self.base_url + '/performance/' + self.write_key)
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
     def submit(self, name, values, delay=None, verbose=None):
         """ Submit prediction scenarios
         :param name:      str         Examples:    cop.json   z1~cop.json   z2~cop~qp.json
@@ -118,7 +155,7 @@ class MicroWriter(MicroReader):
         :return: bool
         """
         verbose = verbose or self.verbose
-        delay = delay or self.delays[0]
+        delay = delay or self.DELAYS[0]
         assert len(values)==self.num_predictions
 
         comma_sep_values = ",".join([ str(v) for v in values ] )
@@ -143,7 +180,7 @@ class MicroWriter(MicroReader):
     def cancel(self, name, delays=None):
         """ Withdraw scenarios """
         codes = list()
-        delays = delays or self.delays
+        delays = delays or self.DELAYS
         for delay in delays:
             res = requests.delete(self.base_url + '/submit/'+name, params={'write_key':self.write_key,'delay':delay} )
             codes.append(res.status_code)
